"""
	cpip
	GameScene classes
"""

import os, ConfigParser
import pygame
from pygame.locals import *
from pygext.gl.all import *

#config = __import__("config")
import config
from object import *
from helper import *
from table import *
from input import *
from gui import *
import gamescene

class GameScene(Scene):
	""" class containing a scene in cpip """
	
	class ImgLayer:
		""" class containing a layer of a scene """
		def __init__(self, imagefile, position):
			self.ok = False
			self.image = Bitmap(imagefile)
			self.position = position
	
	def init(self, name="default", preload=True):
		self.name = name
		self.path = os.path.join(config.DATAPATH, config.SCENEPATH, name)
		self.ok = False
		self.imglayers = {}
		self.preload = preload
		self.ltable = {"self":self}
		self.scriptreturn = []
		self.objects = {}
		self.actor = None
		self.walkmask = None
		self.changeto = None
		if self.preload:
			self.load()
			
	def run_script(self, script):
		self.scriptreturn = []
		if script in self.scripts:
			execfile(self.scripts[script], table.get(), self.ltable)
				
	def handle_keydown(self, event):
		inputman.handle_keydown(event)
		
	def handle_mousemotion(self, event):
		inputman.handle_mousemotion(event)
		
	def handle_mousebuttondown(self, event):
		inputman.handle_mousebuttondown(event)
		
		
	def load(self):
		""" loads the scene with name given by the init funtion """
				
		# open & read config file
		config = ConfigParser.ConfigParser()
		config.readfp(open(os.path.join(self.path, "cfg")))
		
		# list layernames
		self.layernames = []
		for i in config.items("layernames"):
			self.layernames.append(i[1])
			
		wmfile = os.path.join(self.path, config.get("main", "walkmask"))
		self.walkmask = pygame.image.load(wmfile)
		if(config.has_option("main", "positionscale")):
			self.positionscalefactor = config.getfloat("main", "positionscale")
		else:
			self.positionscalefactor = 1.0
			
		# read scriptnames
		self.scripts = {}
		self.scripts = dict(config.items("scripts"))
		for i in self.scripts.keys():
			self.scripts[i] = os.path.join(self.path, self.scripts[i])

		# read & load layers
		self.layers = {}

		for name in self.layernames:
			section = "layer_" + name;
			imgfile = os.path.join(self.path, config.get(section, "image"))
			position = config.getint(section, "pos_x"), config.getint(section, "pos_y")
			z = config.getfloat(section, "pos_z")

			self.imglayers[z] = GameScene.ImgLayer(imgfile, position)
			
		# read and load objects
		for o,g in config.items("objects"):
			self.add_object(o, g)
		
		print "scene", self.name, "loaded"
		self.ok = True
		
	def add_object(self, obj, group = "", setactor = False):
		if isinstance(obj, str):
			self.remove_object(obj)
			self.objects[obj] = Object(name = obj, group = group)
			if setactor:
				self.set_actor(obj)
			return self.objects[obj]
		elif isinstance(obj, Object):
			self.remove_object(obj)
			self.objects[obj.name] = obj
			if setactor:
				self.set_actor(obj.name)
			return obj
		
	def get_object(self, obj):
		if isinstance(obj, Object):
			os = obj.name
		elif isinstance(obj, str):
			os = obj
		if os in self.objects.keys():
			return self.objects[os]
		else:
			return None
		
	def remove_object(self, obj):
		if isinstance(obj, Object):
			os = obj.name
		elif isinstance(obj, str):
			os = obj
		r = self.objects.pop(os, None)
		return r != None
		
	def enter(self):
		if not self.preload:
			self.load()
			
		if not self.ok:
			print "GameScene.enter: not ok"
			return
			
		if inputman.ok == False:
			inputman.enter()
			
		self.new_layer("background", depth = -1.0)
		self.new_layer("zonea", depth = 0.0)
		self.new_layer("foreground", depth = 1.0)
		self.new_layer("text", depth = 2.0)
		self.new_layer("cursor", depth = 3.0)
			
		for z in self.imglayers.keys():
			ilayer = self.imglayers[z]
			ilayer.entity = Entity(ilayer.image)
			if(z < 0.0):
				ilayer.entity.place("background")
			else:
				ilayer.entity.place("foreground")
			ilayer.entity.set(left = ilayer.position[0], top = ilayer.position[1])
			
		for o in self.objects.keys():
			self.objects[o].setup()
		
		self.run_script("enter")
		inputman.show_cursor(True)
		guibase.enter()
					
			
	def exit(self):
		self.run_script("exit")
		for o in self.objects.values():
			o.abort_actions()
		guibase.exit()
		inputman.exit()
		
	def get_scale_on_point(self, pt, absscale, lastscale):
		if self.positionscalefactor == 1.0 or self.walkmask is None:
			return absscale
		try:
			pti = int(pt[0]), int(pt[1])
			pv = self.walkmask.get_at(pti)
		except IndexError:
			return lastscale
		pv = (float(pv[0]) / 128.0) - 1.0
		psf = self.positionscalefactor
		if pv >= 0.0:
			pv = 1.0 + pv * (psf - 1.0)
		else:
			pv = 1.0 + pv * (1.0 - 1.0 / psf)
		return pv * absscale
		
	def is_walkable(self, pt):
		try:
			pti = int(pt[0]), int(pt[1])
			pv = self.walkmask.get_at(pti)
		except IndexError:
			return False
		if pv[0] > 0:
			return True
		else:
			return False		
		
	def get_z(self):
		if not self.ok:
			return -1
		else:
			k = self.layers.keys()
			k.sort()
			return tuple(k)
	
	def check_hotspots(self):
		hovero = None
		for o in self.objects.values():
			if o.hotspot.collidepoint(*inputman.mousepos()):
				if hovero is not None:
					if o.z > hovero.z:
						hovero = o
				else:
					hovero = o
						
		if hovero is None:
			inputman.cursorobject.set_state("default")
		else:
			#inputman.cursorobject.set_state("hotspot")
			if inputman.check("PRICMD"):
				event = "PRICMD"
			elif inputman.check("SECCMD"):
				event = "SECCMD"
			else:
				event = "HOVER" 
			hovero.interact(self.actor, event)
		
	def tick(self):
		inputman.tick()
	
		self.run_script("tick")
		
		self.check_hotspots()
		
		for o in self.objects.keys():
			obj = self.objects[o]
			if obj.changed:
				if obj.get_layer() == "zonea" and obj.never_scale is False:
					fp = obj.x, obj.y
					obj.set(scale =  self.get_scale_on_point(fp, obj.absscale, obj.scale))
			obj.tick()
			
		guibase.tick()
		if self.changeto is not None:
			director.set_scene(gamescene.scenes[self.changeto])
			self.changeto = None
			
	def set_actor(self, actorname):
		if self.actor is not None:
			self.actor.set_actor(False)
			
		if actorname in self.objects.keys():
			self.actor = self.objects[actorname]
			self.actor.set_actor(True)
			
			
scenes = {}

			