""" 
	ceap
	gui class
"""

import os, ConfigParser
import pygame
from pygext.gl.all import *
import pygext.text

from config import *

class GuiBase:
	def __init__(self):
		self.font = None
		self.root = None
		self.panel_alpha = 180
		self.panel_position = (0,0)
		self.panel_border = (5,5)
		self.panel_h = 50
		self.textbuf = []
		self.atext = ["",""]
		self.fade = (0.233, 0.233)
		self.time_per_char = 0.1
		self.started = False
		self.entity = None
		self.t_last_text = 0
		
	def load(self, cfgfile):
		config = ConfigParser.ConfigParser()
		config.readfp(open(os.path.join(DATAPATH, cfgfile)))
		
		fontname, fontsize = config.get("gui", "font").split(",")
		fontsize = int(fontsize)
		self.panel_colour = []
		self.panel_position = []
		self.panel_border = []
		self.panel_alpha = config.getint("gui", "panel_alpha")
		for i in config.get("gui", "panel_position").split(",") : self.panel_position.append(int(i)) 
		self.panel_position = tuple(self.panel_position)
		for i in config.get("gui", "panel_border").split(",") : self.panel_border.append(int(i)) 
		self.panel_border = tuple(self.panel_border)
		pn = config.get("gui", "panel")
		self.panel = Bitmap(os.path.join(DATAPATH, pn))
		
		self.pygamefont = pygame.font.Font(os.path.join(DATAPATH, fontname), fontsize)
		self.font = GLFont(self.pygamefont)
		self.root = director.init_gui()
		
	def clean_output(self):
		self.textbuf = []
		self.t_last_text = director.ticker.now - 1000
		
		
	def enter(self):
		self.clean_output()

		self.entityp = Entity(self.panel)
		self.entityp.set(left = self.panel_position[0], top = self.panel_position[1], alpha = 0)
		self.entityp.place("text")
		self.panel_h = self.entityp.bottom - self.entityp.top
		self.entity1 = TextEntity(font = self.font, text = "")
		self.entity1.set(left = self.entityp.left + self.panel_border[0], top = self.entityp.top + self.panel_border[1])
		self.entity1.place("text")
		self.entity2 = TextEntity(font = self.font, text = "")
		self.entity2.set(left = self.entity1.left, top = self.entity1.top)
		self.entity2.place("text")
		
		
	def exit(self):
		self.entity1.delete()
		self.entity2.delete()
		self.entityp.delete()
		
	def tick(self):
		if self.started and director.ticker.now > self.endon:
			self.started = False
			self.atext = ["",""]
			if len(self.textbuf) ==  0:
				self.t_last_text = director.ticker.now
	
		if self.started is False:
				if len(self.textbuf) >  0:
					self.started = True
					color, self.atext = self.textbuf.pop(0)
					self.entity1.color, self.entity2.color = color
					t = self.split_text(self.pygamefont, self.atext[1], 720)
					lines = len(t)
					t = "\n".join(t)
					ta = self.atext[0]
					if len(ta) > 0:
						ta += ":"
					self.entity2.set_text(t)
					self.entity1.set_text(ta)
					self.entity1.alpha = 0
					self.entity2.alpha = 0
					delay = self.time_per_char * len(t)
					self.endon = director.ticker.now + (delay + self.fade[0] + self.fade[1]) * 1000.0
					taction = AlphaFade(targetalpha = 255, secs = self.fade[0], mode = StopMode) + Delay(delay) + AlphaFade(targetalpha = 0, secs = self.fade[1], mode = StopMode)
					paction = AlphaFade(targetalpha = self.panel_alpha, secs = self.fade[0], mode = StopMode)
					self.entity1.do(taction)
					self.entity2.do(taction)
					self.entityp.do(paction)
					
					w,h = self.pygamefont.size(str(ta + ":"))
					self.entity2.set(left = self.entity1.left + w)
					h *= lines
					h += 2*self.panel_border[1]
					top = self.entityp.top
					self.entityp.set(scale = (1.0, float(h) / float(self.panel_h)))
					self.entityp.set(top = top)
					
		
		if self.t_last_text < director.ticker.now - 1500 and self.t_last_text != 0:
			paction = AlphaFade(targetalpha = 0, secs = self.fade[1], mode = StopMode)
			self.entityp.do(paction)
			self.t_last_text = 0
        		
	def text_out(self, text, color = [(200,200,250), (250,250,250)]):
		if text[0] != self.atext[0] or text[1] != self.atext[1]:
			self.textbuf.append((color, text))
			self.t_last_text = 0
		
	def split_text(self, font, t, maxwidth):
		givenlines = t.split('\n')
		words = []

		for l in givenlines:
			w = l.split(' ')
			words.append('\n')
			words.extend(w)
			
		words.pop(0)
		line = []
		lines = []
		nextword = words.pop(0)
		while True:
			linetext = ' '.join(line+[nextword])
			width,height = font.size(linetext)
			if line and width > maxwidth:
				lines.append(' '.join(line))
				line = []
			elif nextword == "\n":
				lines.append(' '.join(line))
				line = []
				nextword = words.pop(0)
			else:
				line.append(nextword)
				if not words:
					if line:
						lines.append(' '.join(line))
						width,height = font.size(linetext)
					break
				nextword = words.pop(0)
		return lines
        
guibase = GuiBase()