""" 
	ceap
	hotspot class
"""

import pygame

class Hotspot:
	def __init__(self, rects):
		self.rects = rects
		self.d = [0, 0]
		self.scale = 1.0
		for i in range(len(self.rects)):
			if isinstance(self.rects[i], pygame.Rect) is False:
				self.rects[i] = pygame.Rect(self.rects[i])
		
	def move_to(self, pos):
		for r in self.rects:
			r.left = r.left - self.d[0] + pos[0]
			r.top = r.top - self.d[1] + pos[1]
		self.d = pos
		
	def move(self, d):
		for r in self.rects:
			r.left += d[0]
			r.top += d[1]
		self.d[0] += d[0]
		self.d[1] += d[1]
		
	def set_scale(self, scale):
		sf = (1.0 / self.scale) * scale
		self.scale = scale
		for r in self.rects:
			dx = (r.width * sf) - r.width
			dy = (r.height * sf) - r.height
			r.inflate_ip(dx, dy)
			
	def collidepoint(self, x, y):
		for r in self.rects:
			if r.collidepoint(x, y):
				return True
		else:
			return False
			
	def colliderect(self, rect):
		for r in self.rects:
			if r.colliderect(rect):
				return True
		else:
			return False
			
	def collide(self, hotspot):
		for r in self.rects:
			if r.collidelist(hotspot.rects) >= 0:
				return True
		else:
			return False
			
	def get_center(self):
		c = 0
		cx = 0
		cy = 0
		for r in self.rects:
			c += 1
			cx += r.centerx
			cy += r.centery
		return cx / c, cy / c