"""
	ceap
	Object classes
"""

import os, ConfigParser
import pygame
from pygame.locals import *
from pygext.gl.all import *

import config
from state import *
from input import *
from myactions import *
from hotspot import *
from gui import *
import gamescene

class Object(Entity):
	""" class representing an object """
	class Anim:
		def __init__(self, name, frames, spc):
			self.name = name
			self.frames = frames
			self.action = Animate(frames, spc, RepeatMode)
	
	def init(self, name = "None", group = ""):
		self.name = name
		self.group = group
		self.path = os.path.join(config.DATAPATH, config.OBJECTPATH, group, name)
		self.ok = False
		self.alive = False
		self.anims = {}
		self.states = {}
		self.z = -1.0
		self.scripts = {}
		self.astate = None
		self.speed = 0.0
		self.hotspot = None
		self.hotspotpt = (0.5,0.5)
		self.busy = False
		self.actor = False
		self.never_scale = False
		self.ihotspot = None
		self.load()
		
	def load(self):
		self.ok = False
		config = ConfigParser.ConfigParser()
		config.readfp(open(os.path.join(self.path, "cfg")))
		if config.has_option("main", "scale"):
			self.absscale = config.getfloat("main", "scale")
		if config.has_option("main", "never_scale"):
			self.never_scale = config.get("main", "never_scale")
			self.never_scale = bool(self.never_scale)
		astatename = config.get("main", "active_state")
		self.x = config.getint("main", "pos_x")
		self.y = config.getint("main", "pos_y")
		self.z = config.getfloat("main", "pos_z")
		if config.has_option("main", "hotspot_x"):
			self.hotspotpt = config.getfloat("main", "hotspot_x"), config.getfloat("main", "hotspot_y")
		self.scripts = dict(config.items("scripts"))
		for i in self.scripts.keys():
			self.scripts[i] = os.path.join(self.path, self.scripts[i])
		
		i = 0
		while config.has_section("animation_%i"%i):
			section = "animation_%i" % i
			nf = config.getint(section, "frames")
			name = config.get(section, "name")
			extension = ".png"
			if config.has_option(section, "extension"):
				extension = config.get(section, "extension")
			spc = config.getfloat(section, "spc")
			file = os.path.join(self.path, name, "%d" + extension)
			frames = [Bitmap(file % j, self.hotspotpt) for j in range(nf)]
			self.anims[name] = self.Anim(name, frames, spc)
			i += 1
				
		i = 0
		while config.has_section("state_%i"%i):
			section = "state_%i" % i
			name = config.get(section, "name")
			if config.has_option(section, "animation"):
				anim = config.get(section, "animation")
			else:
				anim = None
			if config.has_option(section, "script"):
				script = config.get(section, "script")
			else:
				script = None
			self.states[name] = ObjectState(self, name, anim, script)
			i += 1
			
		if config.has_section("hotspot"):
			rl = []
			i = 0
			while config.has_option("hotspot", "rect_%i"%i):
				s = config.get("hotspot", "rect_%i"%i)
				r = []
				for it in s.split(","): r.append(int(it))
				rl.append(tuple(r))
				i += 1
			self.hotspot = Hotspot(rl)
			
		if self.hotspot is not None:
			if config.has_section("ihotspot"):
				rl = []
				i = 0
				while config.has_option("ihotspot", "rect_%i"%i):
					s = config.get("ihotspot", "rect_%i"%i)
					r = []
					for it in s.split(","): r.append(int(it))
					rl.append(tuple(r))
					i += 1
				self.ihotspot = Hotspot(rl)
			
		self.set_state(astatename)

		self.ok = True
		self.alive = True
		self.changed = True
			
		
	def run_script(self, script, **kw):
		self.scriptreturn = []
		if script in self.scripts:
			execfile(self.scripts[script])
		
	def get_z(self):
		return self.z
		
	def collide_ihotspot(self, actor):
		if self.ihotspot is None or actor is None:
			return False
		if self.ihotspot.collidepoint(actor.x, actor.y):
			return True
		else:
			return False
		
		
		
	def set_state(self, state):
		#print self, "set_state", state
		if state in self.states.keys():
			if self.astate is not None:
				self.astate.exit()
			self.astate = self.states[state]
			self.astate.enter()
		else:
			if self.astate is not None:
				self.astate.exit()
			self.astate = None
		
	def setup(self):
		#self.shape = self.aani.frames[0]
		self.scale = self.absscale
		self.set().place(self.get_layer())

		
	def set(self, **kw):
		self.changed = True
		return Entity.set(self, **kw)
		
	def walk_to(self, x,y):
		if self.busy is True and director.scene.is_walkable((x,y)):
			self.end_actions(Walk)
			self.busy = False
		walk = Walk(x, y, check_start = True, check_tick = True) + CallFunc(self.unbusy)
		self.do(walk)
		self.busy = True
			
	def tick(self):
		self.run_script("tick")
		if self.actor:
			if inputman.check("PRICMD"):
				self.walk_to(*inputman.mousepos())
		
		if self.hotspot is not None:
			self.hotspot.move_to((self.x, self.y))
			self.hotspot.set_scale(self.scale)
		if self.ihotspot is not None:
			self.ihotspot.move_to((self.x, self.y))
			self.ihotspot.set_scale(self.scale)
		if self.astate:
			self.astate.tick()
		
	def get_layer(self):
		if self.z < 0.0:
			return "background"
		elif self.z >= 1.0 and self.z < 2.0:
			return "foreground"
		elif self.z >= 2.0 and self.z < 2.0:
			return "text"
		elif self.z >= 3.0 and self.z < 4.0:
			return "cursor"
		else:
			return "zonea"
			
	def set_actor(self, act):
		self.actor = act
		
	def unbusy(self):
		self.busy = False
			
	def interact(self, actor, cmd):
		self.run_script("interact", actor = actor, cmd = cmd)


			

